@@ -1,8 +1,6 @@
-# -*- coding: utf-8 -*-
-
+"""pytest-pyvista module."""
 
 import os
-import pathlib
 import platform
 import warnings
 
@@ -28,6 +26,21 @@ def pytest_addoption(parser):
         action="store_true",
         help="Enables failure if image cache does not exist.",
     )
+    group.addoption(
+        "--generated_image_dir",
+        action="store",
+        help="Path to dump test images from the current run.",
+    )
+    group.addoption(
+        "--add_missing_images",
+        action="store_true",
+        help="Adds images to cache if missing.",
+    )
+    group.addoption(
+        "--skip_image_cache_vtk8",
+        action="store_true",
+        help="Skips image cache testing for vtk8.",
+    )
     group.addoption(
         "--image_cache_dir",
         action="store",
@@ -52,41 +65,71 @@ class VerifyImageCache:
     Parameters
     ----------
     test_name : str
-        Name of test to save.  It is used to define the name of image cache file.
+        Name of test to save.  It is used to define the name of image cache
+        file.
 
-    error_value : int
-        Threshold value for determining if two images are not similar enough in a test.
+    cache_dir : str
+        Directory for image cache comparisons.
 
-    warning_value : int
-        Threshold value to warn that two images are different but not enough to fail the test.
+    error_value : float, default: 500
+        Threshold value for determining if two images are not similar enough in
+        a test.
 
-    var_error_value : int
+    warning_value : float, default: 200
+        Threshold value to warn that two images are different but not enough to
+        fail the test.
+
+    var_error_value : float, default: 1000
         Same as error_value but for high variance tests.
 
-    var_warning_value : int
-        same as warning_value but for high variance tests.
+    var_warning_value : float, default 1000
+        Same as warning_value but for high variance tests.
+
+    generated_image_dir : str, optional
+        Directory to save generated images.  If not specified, no generated
+        images are saved.
+
+    Examples
+    --------
+    Create an image cache directory named image_cache and check a simple
+    plotter against it. Since ``image_cache`` doesn't exist, it will be created
+    and basic.png will be added to it. Subsequent calls to ``verif`` will
+    compare the plotter against the cached image.
+
+    >>> import pyvista as pv
+    >>> from pytest_pyvista import VerifyImageCache
+    >>> pl = pv.Plotter(off_screen=True)
+    >>> pl.add_mesh(pv.Sphere())
+    >>> verif = VerifyImageCache('test_basic', 'image_cache')
+    >>> verif(pl)
+
     """
 
     reset_image_cache = False
     ignore_image_cache = False
     fail_extra_image_cache = False
+    add_missing_images = False
+    skip_image_cache_vtk8 = False
 
     def __init__(
         self,
         test_name,
         cache_dir,
         *,
-        error_value=500,
-        warning_value=200,
-        var_error_value=1000,
-        var_warning_value=1000,
+        error_value=500.0,
+        warning_value=200.0,
+        var_error_value=1000.0,
+        var_warning_value=1000.0,
+        generated_image_dir=None,
     ):
         self.test_name = test_name
 
         self.cache_dir = cache_dir
 
         if not os.path.isdir(self.cache_dir):
-            warnings.warn(f"pyvista test cache image dir: {self.cache_dir} does not yet exist'  Creating empty cache.")
+            warnings.warn(
+                f"pyvista test cache image dir: {self.cache_dir} does not yet exist'  Creating empty cache."
+            )
             os.mkdir(self.cache_dir)
 
         self.error_value = error_value
@@ -94,6 +137,15 @@ class VerifyImageCache:
         self.var_error_value = var_error_value
         self.var_warning_value = var_warning_value
 
+        self.generated_image_dir = generated_image_dir
+        if self.generated_image_dir is not None and not os.path.isdir(
+            self.generated_image_dir
+        ):
+            warnings.warn(
+                f"pyvista test generated image dir: {self.generated_image_dir} does not yet exist.  Creating dir."
+            )
+            os.makedirs(self.generated_image_dir)
+
         self.high_variance_test = False
         self.windows_skip_image_cache = False
         self.macos_skip_image_cache = False
@@ -114,7 +166,10 @@ class VerifyImageCache:
             return
 
         if not VTK9:
-            raise RuntimeError("Image cache is only valid for VTK9+")
+            if self.skip_image_cache_vtk8:
+                return
+            else:
+                raise RuntimeError("Image cache is only valid for VTK9+")
 
         if self.ignore_image_cache:
             return
@@ -142,16 +197,19 @@ class VerifyImageCache:
         # cached image name. We remove the first 5 characters of the function name
         # "test_" to get the name for the image.
         image_filename = os.path.join(self.cache_dir, test_name[5:] + ".png")
-
         if not os.path.isfile(image_filename) and self.fail_extra_image_cache:
             raise RuntimeError(f"{image_filename} does not exist in image cache")
-        # simply save the last screenshot if it doesn't exist or the cache
-        # is being reset.
-        if self.reset_image_cache or not os.path.isfile(image_filename):
-            return plotter.screenshot(image_filename)
 
-        # otherwise, compare with the existing cached image
+        if self.add_missing_images and not os.path.isfile(image_filename):
+            plotter.screenshot(image_filename)
+
+        if self.generated_image_dir is not None:
+            gen_image_filename = os.path.join(
+                self.generated_image_dir, test_name[5:] + ".png"
+            )
+            plotter.screenshot(gen_image_filename)
         error = pyvista.compare_images(image_filename, plotter)
+
         if error > allowed_error:
             raise RuntimeError(
                 f"{test_name} Exceeded image regression error of "
@@ -175,11 +233,20 @@ def verify_image_cache(request, pytestconfig):
     VerifyImageCache.fail_extra_image_cache = pytestconfig.getoption(
         "fail_extra_image_cache"
     )
+    VerifyImageCache.add_missing_images = pytestconfig.getoption("add_missing_images")
+    VerifyImageCache.skip_image_cache_vtk8 = pytestconfig.getoption(
+        "skip_image_cache_vtk8"
+    )
 
     cache_dir = pytestconfig.getoption("image_cache_dir")
     if cache_dir is None:
         cache_dir = pytestconfig.getini("image_cache_dir")
 
-    verify_image_cache = VerifyImageCache(request.node.name, cache_dir)
+    gen_dir = pytestconfig.getoption("generated_image_dir")
+
+    verify_image_cache = VerifyImageCache(
+        request.node.name, cache_dir, generated_image_dir=gen_dir
+    )
     pyvista.global_theme.before_close_callback = verify_image_cache
+
     return verify_image_cache
